@@ -1,24 +1,27 @@
 # -*- coding:utf-8 -*-
 from datetime import datetime
 from datetime import timedelta
+from time import time
 from zope import interface
 from zope import component
 from Products.CMFPlone import utils
 from Products.Five import BrowserView
 from zope.interface import implements
 from Acquisition import aq_inner
-from Products.CMFPlone.utils import getToolByName,log
+from Products.CMFPlone.utils import getToolByName
 
 from sqlalchemy import sql, orm
 from sqlalchemy import and_
 from sqlalchemy import func 
 from sqlalchemy.sql.expression import desc
 
-from plone.memoize.instance import memoize
+from plone.memoize import instance, ram
 
 from sc.social.viewcounter.pageview import session
 from sc.social.viewcounter.pageview import PageView
 
+import logging
+logger = logging.getLogger('sc.social.viewcounter:')
 
 class PageViewBV(BrowserView):
     """ Base BrowserView to PageView
@@ -76,10 +79,14 @@ class Reports(BrowserView):
         self.blacklisted_types = blacklisted_types
         super(Reports, self).__init__(context, request)
     
-    def _reportPageViews(self,dateRange,portal_type='',path='',**kw):
+    
+    @ram.cache(lambda *args: time() // (60 * 60))
+    def _reportPageViews(self,dateRange=None,portal_type='',path='',**kw):
+        logger.debug('Query sql.')
         session = self.session
         where=[]
-        where.append(PageView.access_datetime.between(*dateRange))
+        if dateRange:
+            where.append(PageView.access_datetime.between(*dateRange))
         if portal_type:
             where.append(PageView.object_type==portal_type)
         if path:
@@ -104,8 +111,8 @@ class Reports(BrowserView):
                                       if brain.portal_type not in blacklisted])
         return dictByUIDs
         
-    
     @property
+    @ram.cache(lambda *args: time() // (60 * 60))
     def lastHour(self):
         now = datetime.now()
         delta = timedelta(0,3600)
@@ -127,6 +134,7 @@ class Reports(BrowserView):
         return (start,end)
     
     @property
+    @ram.cache(lambda *args: time() // (60 * 60))
     def lastDay(self):
         now = datetime.now()
         delta = timedelta(1,0)
@@ -148,6 +156,7 @@ class Reports(BrowserView):
         return (start,end)
     
     @property
+    @ram.cache(lambda *args: time() // (60 * 60 * 24))
     def lastWeek(self):
         now = datetime.now()
         weekday = now.weekday()
@@ -170,9 +179,10 @@ class Reports(BrowserView):
                        59,
                        )
         return (start,end)
-        
+    
     
     @property
+    @ram.cache(lambda *args: time() // (60 * 60 * 24))
     def lastMonth(self):
         now = datetime.now()
         day = now.day
@@ -196,7 +206,7 @@ class Reports(BrowserView):
         return (start,end)
         
     
-    @property
+    @instance.memoizedproperty
     def allTime(self):
         now = datetime.now()
         base_dt = now
@@ -211,9 +221,10 @@ class Reports(BrowserView):
         return (start,end)
         
     
-    @property
+    @instance.memoizedproperty
     def lastYear(self):
         now = datetime.now()
+        weekday = now.weekday()
         base_dt = now
         start = datetime(base_dt.year-1,
                          1,
@@ -234,30 +245,35 @@ class Reports(BrowserView):
         return (start,end)
         
     
+    def cleanKw(self,kw):
+        newKw = {}
+        for k,v in kw.items():
+            if isinstance(v,list):
+                v = tuple(v)
+            newKw[k]=v
+        return newKw
+    
     def viewsLastHour(self,**kw):
         range = self.lastHour
-        return self._reportPageViews(range,**kw)
+        return self._reportPageViews(range, **self.cleanKw(kw))
     
     def viewsLastDay(self,**kw):
         range = self.lastDay
-        return self._reportPageViews(range,**kw)
+        return self._reportPageViews(range, **self.cleanKw(kw))
     
     def viewsLastWeek(self,**kw):
         range = self.lastWeek
-        return self._reportPageViews(range,**kw)
+        return self._reportPageViews(range, **self.cleanKw(kw))
     
     def viewsLastMonth(self,**kw):
         range = self.lastMonth
-        return self._reportPageViews(range,**kw)
+        return self._reportPageViews(range, **self.cleanKw(kw))
     
     def viewsLastYear(self,**kw):
         range = self.lastYear
-        return self._reportPageViews(range,**kw)
+        return self._reportPageViews(range, **self.cleanKw(kw))
     
     def viewsAllTime(self,**kw):
-        range = self.allTime
-        return self._reportPageViews(range,**kw)
-    
-
-
-       
\ No newline at end of file
+        # If we really want everything, let's do it :-)
+        range = None
+        return self._reportPageViews(range, **self.cleanKw(kw))
\ No newline at end of file
